@@ -28,7 +28,6 @@
 #include <QtNetwork/QNetworkRequest>
 #include <QtCore/QProcess>
 #include <QtCore/QTimer>
-#include <QtCore/QUrl>
 #include <QtGui/QMessageBox>
 
 #include "GeoSceneDocument.h"
@@ -76,22 +75,18 @@ ControlView::ControlView( QWidget *parent )
 
     m_control->addMarbleWidget( m_marbleWidget );
 
-    // TODO: Creating a second MapThemeManager may not be the best solution here.
-    // MarbleModel also holds one with a QFileSystemWatcher.
-    m_mapThemeManager = new MapThemeManager;
-    
     connect( m_control, SIGNAL( showMapWizard() ), this, SIGNAL( showMapWizard() ) );
     connect( m_control, SIGNAL( showUploadDialog() ), this, SIGNAL( showUploadDialog() ) );
 }
 
 ControlView::~ControlView()
 {
-    delete m_mapThemeManager;
+    // nothing to do
 }
 
 QString ControlView::applicationVersion()
 {
-    return "1.1.1 (stable version)";
+    return "1.1.90 (1.2 RC 1)";
 }
 
 void ControlView::zoomIn()
@@ -169,7 +164,7 @@ QString ControlView::defaultMapThemeId() const
     }
 
     QStringList installedThemes;
-    QList<GeoSceneDocument const*> themes = m_mapThemeManager->mapThemes();
+    QList<GeoSceneDocument const*> themes = m_marbleWidget->model()->mapThemeManager()->mapThemes();
     foreach(GeoSceneDocument const* theme, themes) {
         installedThemes << theme->head()->mapThemeId();
     }
@@ -206,11 +201,11 @@ void ControlView::printMapScreenShot( QPointer<QPrintDialog> printDialog)
             QTextDocument document;
             QString text = "<html><head><title>Marble Printout</title></head><body>";
             QPalette const originalPalette = m_marbleWidget->palette();
-            bool const wasBackgroundVisible = m_marbleWidget->map()->showBackground();
+            bool const wasBackgroundVisible = m_marbleWidget->showBackground();
             bool const hideBackground = !mapCoversViewport && !printOptions->printBackground();
             if ( hideBackground ) {
                 // Temporarily remove the black background and layers painting on it
-                m_marbleWidget->map()->setShowBackground( false );
+                m_marbleWidget->setShowBackground( false );
                 m_marbleWidget->setPalette( QPalette ( Qt::white ) );
                 m_marbleWidget->repaint();
             }
@@ -240,7 +235,7 @@ void ControlView::printMapScreenShot( QPointer<QPrintDialog> printDialog)
             document.print( printDialog->printer() );
 
             if ( hideBackground ) {
-                m_marbleWidget->map()->setShowBackground( wasBackgroundVisible );
+                m_marbleWidget->setShowBackground( wasBackgroundVisible );
                 m_marbleWidget->setPalette( originalPalette );
                 m_marbleWidget->repaint();
             }
@@ -346,7 +341,7 @@ void ControlView::printRouteSummary( QTextDocument &document, QString &text)
         }
 
         QString label = "<p>%1 %2</p>";
-        qreal distance = routingModel->totalDistance();
+        qreal distance = routingModel->route().distance();
         QString unit = distance > 1000 ? "km" : "m";
         int precision = distance > 1000 ? 1 : 0;
         if ( distance > 1000 ) {
@@ -383,55 +378,43 @@ void ControlView::printDrivingInstructions( QTextDocument &document, QString &te
         return;
     }
 
-    GeoDataLineString total;
-    for ( int i=0; i<routingModel->rowCount(); ++i ) {
-        QModelIndex index = routingModel->index(i, 0);
-        RoutingModel::RoutingItemType type = qVariantValue<RoutingModel::RoutingItemType>(index.data(RoutingModel::TypeRole));
-        if ( type == RoutingModel::WayPoint ) {
-            GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>(index.data(RoutingModel::CoordinateRole));
-            total.append(coordinates);
-        }
-    }
+    GeoDataLineString total = routingModel->route().path();
 
     text += "<table cellpadding=\"4\">";
     text += "<tr><th>No.</th><th>Distance</th><th>Instruction</th></tr>";
-    for ( int i=0, j=0; i<routingModel->rowCount(); ++i ) {
+    for ( int i=0; i<routingModel->rowCount(); ++i ) {
         QModelIndex index = routingModel->index(i, 0);
-        RoutingModel::RoutingItemType type = qVariantValue<RoutingModel::RoutingItemType>( index.data( RoutingModel::TypeRole ) );
-        if ( type == RoutingModel::Instruction ) {
-            ++j;
-            GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>( index.data( RoutingModel::CoordinateRole ) );
-            GeoDataLineString accumulator;
-            for (int k=0; k<total.size(); ++k) {
-                accumulator << total.at(k);
+        GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>( index.data( MarblePlacemarkModel::CoordinateRole ) );
+        GeoDataLineString accumulator;
+        for (int k=0; k<total.size(); ++k) {
+            accumulator << total.at(k);
 
-                if (total.at(k) == coordinates)
-                    break;
-            }
-
-            if ( i%2 == 0 ) {
-                text += "<tr bgcolor=\"lightGray\"><td align=\"right\" valign=\"middle\">";
-            }
-            else {
-                text += "<tr><td align=\"right\" valign=\"middle\">";
-            }
-            text += QString::number( j );
-            text += "</td><td align=\"right\" valign=\"middle\">";
-
-            text += QString::number( accumulator.length( EARTH_RADIUS ) * METER2KM, 'f', 1 );
-            /** @todo: support localization */
-            text += " km</td><td valign=\"middle\">";
-
-            QPixmap instructionIcon = qVariantValue<QPixmap>( index.data( Qt::DecorationRole ) );
-            if ( !instructionIcon.isNull() ) {
-                QString uri = QString("marble://turnIcon%1.png").arg(i);
-                document.addResource( QTextDocument::ImageResource, QUrl( uri ), QVariant( instructionIcon ) );
-                text += QString("<img src=\"%1\">").arg(uri);
-            }
-
-            text += routingModel->data( index ).toString();
-            text += "</td></tr>";
+            if (total.at(k) == coordinates)
+                break;
         }
+
+        if ( i%2 == 0 ) {
+            text += "<tr bgcolor=\"lightGray\"><td align=\"right\" valign=\"middle\">";
+        }
+        else {
+            text += "<tr><td align=\"right\" valign=\"middle\">";
+        }
+        text += QString::number( i+1 );
+        text += "</td><td align=\"right\" valign=\"middle\">";
+
+        text += QString::number( accumulator.length( EARTH_RADIUS ) * METER2KM, 'f', 1 );
+        /** @todo: support localization */
+        text += " km</td><td valign=\"middle\">";
+
+        QPixmap instructionIcon = qVariantValue<QPixmap>( index.data( Qt::DecorationRole ) );
+        if ( !instructionIcon.isNull() ) {
+            QString uri = QString("marble://turnIcon%1.png").arg(i);
+            document.addResource( QTextDocument::ImageResource, QUrl( uri ), QVariant( instructionIcon ) );
+            text += QString("<img src=\"%1\">").arg(uri);
+        }
+
+        text += routingModel->data( index ).toString();
+        text += "</td></tr>";
     }
     text += "</table>";
 #endif
@@ -550,6 +533,16 @@ bool ControlView::setSideBarState( const QByteArray &state )
     return m_splitter ? m_splitter->restoreState( state ) : false;
 }
 
+void ControlView::addGeoDataFile( QString filename )
+{
+    m_marbleWidget->model()->addGeoDataFile( filename );
+}
+
+SunLocator* ControlView::sunLocator()
+{
+    return m_marbleWidget->model()->sunLocator();
+}
+
 }
 
 #include "ControlView.moc"
