@@ -25,7 +25,8 @@
 #include <QtGui/QToolTip>
 
 #if QT_VERSION >= 0x40600
-  #include <QtGui/QTouchEvent>
+  #include <QtGui/QGestureEvent>
+  #include <QtGui/QPinchGesture>
 #endif
 
 #include "global.h"
@@ -33,7 +34,6 @@
 #include "GeoDataCoordinates.h"
 #include "MarbleDirs.h"
 #include "MarbleWidget.h"
-#include "MarbleMap.h"
 #include "MarbleModel.h"
 #include "ViewportParams.h"
 #include "AbstractFloatItem.h"
@@ -50,19 +50,19 @@ const int TOOLTIP_START_INTERVAL = 1000;
 class MarbleWidgetInputHandler::Protected
 {
 public:
-    Protected();
+    Protected( MarbleWidget *widget );
 
-    MarbleWidget *m_widget;
-    MarbleModel *m_model;
+    MarbleWidget *const m_widget;
+    MarbleModel *const m_model;
     bool m_positionSignalConnected;
     QTimer *m_mouseWheelTimer;
     Qt::MouseButtons m_disabledMouseButtons;
     qreal m_wheelZoomTargetDistance;
 };
 
-MarbleWidgetInputHandler::Protected::Protected()
-    : m_widget( 0 ),
-      m_model( 0 ),
+MarbleWidgetInputHandler::Protected::Protected( MarbleWidget *widget )
+    : m_widget( widget ),
+      m_model( widget->model() ),
       m_positionSignalConnected( false ),
       m_mouseWheelTimer( 0 ),
       m_disabledMouseButtons( Qt::NoButton ),
@@ -71,12 +71,20 @@ MarbleWidgetInputHandler::Protected::Protected()
 }
 
 
-MarbleWidgetInputHandler::MarbleWidgetInputHandler()
-    : d( new Protected )
+MarbleWidgetInputHandler::MarbleWidgetInputHandler( MarbleWidget *widget )
+    : d( new Protected( widget ) )
 {
     d->m_mouseWheelTimer = new QTimer( this );
     connect( d->m_mouseWheelTimer, SIGNAL( timeout() ),
 	     this, SLOT( restoreViewContext() ) );
+    
+    connect( d->m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
+             this,        SLOT( installPluginEventFilter( RenderPlugin * ) ) );
+
+    foreach( RenderPlugin *renderPlugin, d->m_widget->renderPlugins() ) {
+        if( renderPlugin->isInitialized() )
+            d->m_widget->installEventFilter( renderPlugin );
+    }
 }
 
 MarbleWidgetInputHandler::~MarbleWidgetInputHandler()
@@ -95,20 +103,6 @@ bool MarbleWidgetInputHandler::isPositionSignalConnected() const
     return d->m_positionSignalConnected;
 }
 
-void MarbleWidgetInputHandler::init( MarbleWidget *w )
-{
-    d->m_widget = w;
-    d->m_model = w->model();
-    
-    connect( d->m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
-             this,        SLOT( installPluginEventFilter( RenderPlugin * ) ) );
-
-    foreach( RenderPlugin *renderPlugin, d->m_widget->renderPlugins() ) {
-        if( renderPlugin->isInitialized() )
-            d->m_widget->installEventFilter( renderPlugin );
-    }
-}
-
 void MarbleWidgetInputHandler::setMouseButtonPopupEnabled( Qt::MouseButton mouseButton, bool enabled )
 {
     if ( enabled ) {
@@ -247,7 +241,7 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
     soon.setPlanetAxis(now->planetAxis());
     soon.setSize(now->size());
 
-    qreal newRadius = marbleWidget->map()->radiusFromDistance(newDistance);
+    qreal newRadius = marbleWidget->radiusFromDistance(newDistance);
     soon.setRadius( newRadius );
 
     qreal mouseLon, mouseLat;
@@ -291,8 +285,8 @@ void MarbleWidgetDefaultInputHandler::Private::MoveTo( MarbleWidget* marbleWidge
     marbleWidget->flyTo( lookAt );
 }
 
-MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler()
-    : MarbleWidgetInputHandler(), d( new Private )
+MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler( MarbleWidget *widget )
+    : MarbleWidgetInputHandler( widget ), d( new Private )
 {
     d->m_selectionRubber = new QRubberBand( QRubberBand::Rectangle,
                                             MarbleWidgetInputHandler::d->m_widget );
@@ -303,41 +297,6 @@ MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler()
     connect( &d->m_toolTipTimer, SIGNAL( timeout() ), this, SLOT( openItemToolTip() ) );
     d->m_lmbTimer.setSingleShot(true);
     connect( &d->m_lmbTimer, SIGNAL(timeout()), this, SLOT(lmbTimeout()));
-}
-
-MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
-{
-    // FIXME: move to Private
-    delete d->m_selectionRubber;
-    delete d;
-}
-
-void MarbleWidgetDefaultInputHandler::lmbTimeout()
-{
-    if (!d->m_selectionRubber->isVisible()) {
-        emit lmbRequest( d->m_leftPressedX, d->m_leftPressedY );
-    }
-}
-
-void MarbleWidgetInputHandler::restoreViewContext()
-{
-    // Needs to stop the timer since it repeats otherwise.
-    d->m_mouseWheelTimer->stop();
-
-    // Redraw the map with the quality set for Still (if necessary).
-    d->m_widget->setViewContext( Still );
-    d->m_widget->viewport()->resetFocusPoint();
-    d->m_wheelZoomTargetDistance = 0.0;
-}
-
-void MarbleWidgetInputHandler::installPluginEventFilter( RenderPlugin *renderPlugin )
-{
-    d->m_widget->installEventFilter( renderPlugin );
-}
-
-void MarbleWidgetDefaultInputHandler::init( MarbleWidget *w )
-{
-    MarbleWidgetInputHandler::init( w );
 
     // The interface to the measure tool consists of a RMB popup menu
     // and some signals.
@@ -375,6 +334,36 @@ void MarbleWidgetDefaultInputHandler::init( MarbleWidget *w )
              this, SLOT( showLmbMenu( int, int ) ) );
 }
 
+MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
+{
+    // FIXME: move to Private
+    delete d->m_selectionRubber;
+    delete d;
+}
+
+void MarbleWidgetDefaultInputHandler::lmbTimeout()
+{
+    if (!d->m_selectionRubber->isVisible()) {
+        emit lmbRequest( d->m_leftPressedX, d->m_leftPressedY );
+    }
+}
+
+void MarbleWidgetInputHandler::restoreViewContext()
+{
+    // Needs to stop the timer since it repeats otherwise.
+    d->m_mouseWheelTimer->stop();
+
+    // Redraw the map with the quality set for Still (if necessary).
+    d->m_widget->setViewContext( Still );
+    d->m_widget->viewport()->resetFocusPoint();
+    d->m_wheelZoomTargetDistance = 0.0;
+}
+
+void MarbleWidgetInputHandler::installPluginEventFilter( RenderPlugin *renderPlugin )
+{
+    d->m_widget->installEventFilter( renderPlugin );
+}
+
 void MarbleWidgetDefaultInputHandler::showLmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::LeftButton ) ) {
@@ -731,41 +720,28 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             qreal target = MarbleWidgetInputHandler::d->m_wheelZoomTargetDistance;
             if ( marbleWidget->animationsEnabled() && target > 0.0 ) {
                 // Do not use intermediate (interpolated) distance values caused by animations
-                zoom = marbleWidget->map()->zoomFromDistance( target );
+                zoom = marbleWidget->zoomFromDistance( target );
             }
-            qreal newDistance = marbleWidget->map()->distanceFromZoom( zoom + steps );
+            qreal newDistance = marbleWidget->distanceFromZoom( zoom + steps );
             MarbleWidgetInputHandler::d->m_wheelZoomTargetDistance = newDistance;
             d->ZoomAt(MarbleWidgetInputHandler::d->m_widget, wheelevt->pos(), newDistance);
 
             MarbleWidgetInputHandler::d->m_mouseWheelTimer->start( 400 );
             return true;
         }
-#if QT_VERSION >= 0x40600
-        else if ( e->type() == QEvent::TouchBegin ||
-                  e->type() == QEvent::TouchUpdate ||
-                  e->type() == QEvent::TouchEnd) {
-
-            QList<QTouchEvent::TouchPoint> touchPoints = static_cast<QTouchEvent *>( e )->touchPoints();
-            if (touchPoints.count() == 2) {
-                const QTouchEvent::TouchPoint &touchPoint0 = touchPoints.first();
-                const QTouchEvent::TouchPoint &touchPoint1 = touchPoints.last();
-                const QLineF line0(touchPoint0.lastPos(), touchPoint1.lastPos());
-                const QLineF line1(touchPoint0.pos(), touchPoint1.pos());
-                //scaleFactor is the ratio the view will be scaled compared to now,
-                // 1:the same, 2: the double, 0.5: half
-                qreal scaleFactor = 1;
-
-                if (line0.length() > 0) {
-                    scaleFactor = line1.length() / line0.length();
-                }
+        else if ( e->type() == QEvent::Gesture ) {
+            QGestureEvent *ge = static_cast<QGestureEvent *>(e);
+            QPinchGesture *pinch = static_cast<QPinchGesture*>(ge->gesture(Qt::PinchGesture));
+            if (pinch) {
+                qreal scaleFactor = pinch->scaleFactor();
+                qreal  destLat;
+                qreal  destLon;
+                QPointF center = pinch->centerPoint();
 
                 MarbleWidget *marbleWidget = MarbleWidgetInputHandler::d->m_widget;
 
                 marbleWidget->setViewContext( Animation );
 
-                qreal  destLat;
-                qreal  destLon;
-                QPointF center = line1.pointAt(0.5);
                 bool isValid = marbleWidget->geoCoordinates(center.x(), center.y(),
                              destLon, destLat, GeoDataCoordinates::Radian );
 
@@ -774,11 +750,10 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 }
 
                 //convert the scaleFactor to be 0: the same: < 0: smaller, > 0: bigger and make it bigger by multiplying for an arbitrary big value
-                marbleWidget->zoomViewBy( (scaleFactor-1)*200);
-                MarbleWidgetInputHandler::d->m_mouseWheelTimer->start( 400 );
+               marbleWidget->zoomViewBy( (scaleFactor-1)*200); 
+               return true;
             }
         }
-#endif
         else
             return false;
     }
